#!/bin/bash
# ~/bin/example
. $(ffoom path)


##
## pretty.sh - make output pretty and pretty useful
##

ffoo import pretty


# three levels should be enough for everyone
#
think "about saying something..."   # thinks loud if FFOO_VERBOSE
warn "this is just an example"      # obvious :)
true || die "true was false!"       # Perlism :)


# several ways of debugging
#
x=1; y=2; z=3
debug -v x y z                      # debug:example.sh:x='1' ...
echo "hello" | debug_pipe mypipe    # debug|mypipe|: hello
warn -f should_be_empty             # warns with name+content
                                    # unless the file is empty
myfun() {
    num=42
    debug -v num                    # reveals context!
}                                   #       vvv
myfun                               # debug:myfun: num='42'


# but you can throw more at yourself
#
debug -c ps aux                     # when more is more
debug -f /etc/hosts

# output can be whatever you think is pretty:
#
FFOO_PRETTY=color
FFOO_PRETTY=html
#...
FFOO_PRETTY=your_own_format



##
## config.sh -- config file grepping
##

ffoo import config

# a sample INI file
#
mkdir $HOME/.demo
cat >$HOME/.demo/foo.ini <<EOF
# a sample INI file
[foo]
    multiline = also possible
    multiline = like this :)
[foo.bar]
    # spaces after equal sign are trimmed by default
    baz     =   qux
    baz2    =   public qux
[foo.script]
    # but can be preserved using -S|--strict mode
    s1 =#!/usr/bin/python
    s1 =def main():
    s1 =    print "will work!"
EOF

# read as you need
#
FFOO_CONFIG_PATH=$HOME/.demo            # just tell us the path
inigrep -s foo.bar -k baz foo.ini       # prints "qux"
inigrep -p foo.bar.baz foo.ini          # ...short for above
inigrep -p foo.bar.baz                  # ...even shorter (foo hints foo.ini)

# store even indented text
#
inigrep -S -p foo.script python >s1.py  # the -S will preserve spaces on RHS
python s1.py                            # so this will work


# merge INIs on runtime
#
mkdir $HOME/.demo/private
cat >$HOME/.demo/private/foo.ini <<EOF
[foo.bar]
    baz2 = secret qux
EOF

# this works similar to PATH
#
FFOO_CONFIG_PATH=$HOME/.demo-private:$HOME/.demo
inigrep    -p foo.bar.baz2              # the private file wins -- "secret qux!"
inigrep -j -p foo.bar.baz2              # but -j causes them to be joined:
                                        #   secret qux
                                        #   public qux
